unit Root;
interface
uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  rijndael,
  TeleToken,
  TeleTokenAPI,
  Glob,
  GlobErr,
  AppEvnts,
  AnsiStrings;

type
  TfrmRoot = class(TForm)
    btOpen: TButton;
    btClose: TButton;
    btRAMWrite: TButton;
    btMemWrite: TButton;
    btMemRead: TButton;
    btMemWProtect: TButton;
    btMemProtect: TButton;
    btCounterWrite: TButton;
    btGetFeature: TButton;
    btRAMRead: TButton;
    btCounterRead: TButton;
    btCounterIncRead: TButton;
    btCounterDecRead: TButton;
    btCounterConfigWrite: TButton;
    btCounterConfigSet: TButton;
    btCounterConfigClear: TButton;
    btPassWrite: TButton;
    btPassConfigWrite: TButton;
    btPassConfigSet: TButton;
    PassConfigClear: TButton;
    btLosuj: TButton;
    btCounterIncEncryptRead: TButton;
    btEncrypt: TButton;
    btDecrypt: TButton;
    btEncryptBlock: TButton;
    btDecryptBlock: TButton;
    Memo: TMemo;
    OpenDlg: TOpenDialog;
    ApplicationEvents: TApplicationEvents;
    function OpenSession(): integer;
    function CloseSession(): integer;
    procedure btOpenClick(Sender: TObject);
    procedure btGetFeatureClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure btRAMWriteClick(Sender: TObject);
    procedure btRAMReadClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btMemWriteClick(Sender: TObject);
    procedure btMemReadClick(Sender: TObject);
    procedure btMemWProtectClick(Sender: TObject);
    procedure btMemProtectClick(Sender: TObject);
    procedure btCounterWriteClick(Sender: TObject);
    procedure btCounterReadClick(Sender: TObject);
    procedure btCounterIncReadClick(Sender: TObject);
    procedure btCounterDecReadClick(Sender: TObject);
    procedure btCounterIncEncryptReadClick(Sender: TObject);
    procedure btCounterConfigWriteClick(Sender: TObject);
    procedure btCounterConfigSetClick(Sender: TObject);
    procedure btCounterConfigClearClick(Sender: TObject);
    procedure btPassWriteClick(Sender: TObject);
    procedure btPassConfigWriteClick(Sender: TObject);
    procedure btPassConfigSetClick(Sender: TObject);
    procedure PassConfigClearClick(Sender: TObject);
    procedure btEncryptClick(Sender: TObject);
    procedure btDecryptClick(Sender: TObject);
    procedure btEncryptBlockClick(Sender: TObject);
    procedure btDecryptBlockClick(Sender: TObject);
    procedure btLosujClick(Sender: TObject);
    procedure ApplicationEventsMessage(var Msg: tagMSG;
      var Handled: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmRoot: TfrmRoot;

implementation

{$R *.dfm}
//---------------------------------------------------------------------------
procedure TfrmRoot.FormCreate(Sender: TObject);
var
  n     : byte;
  stmp  : string;
begin
  //wywoanie funkcji inicjalizujc bibliotek TeleTokenHi
  TVTTAPI_Initialize();

  //utworzenie tablicy hase dla TeleTokenu
  n:=0;
  while (n<PASS_NO) do
    begin
      stmp:='Pass' + IntToStr(n);
      AnsiStrings.StrCopy(@PassTbl[n*16], PAnsiChar(AnsiString(stmp)));
      inc(n);
    end;
end;
//---------------------------------------------------------------------------
function TfrmRoot.OpenSession(): integer;
var
  sErr  : string;
begin
  sErr := 'StartSession error' + CRLF + '--------------------';

  // sprawdzenie czy uchwyt na TeleToken nie jest wyzerowany
  if (hCon=nil) then
    begin
      Memo.Lines.Add(sErr);
      result:=-1;
      exit;
    end;

  // prba otwarcia sesji
  NumPassT:=TVTTAPI_StartSession(Pass, hCon, 1);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<> 0) then
    begin
      Memo.Lines.Add(sErr);
      result:=-1;
      exit;
    end;

  // obliczenie hase dla rozkazu sesji
  TVTTAPI_CalcPass(Pass, hCon, PassTbl[16], PassTbl[NumPassT * 16]);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<>0) then
    begin
      result:=-1;
      exit;
    end;
  result:=0;
end;
//---------------------------------------------------------------------------
function TfrmRoot.CloseSession(): integer;
begin
  // zamknicie sesji
  TVTTAPI_EndSession(Pass, hCon);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<>0) then
    begin
      Memo.Lines.Add('EndSession error' + CRLF + '--------------------');
      result:=-1;
      exit;
    end;

  result:=0;
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btOpenClick(Sender: TObject);
begin
  // sprawdzenie czy otwarto wczeniej TeleToken
  if (hCon<>nil) then
  begin
    Memo.Lines.Add('TeleToken jest ju otwarty' + CRLF + '--------------------');
    exit;
  end;

  // losowanie hase dla sesji
  TVTTAPI_Rand(Pass);

  Memo.Lines.Add('Open' + CRLF);

  // otwarcie pierwszego TeleTokenu o ID zapisanym w TOKEN_ID
  hCon:=TVTTAPI_Open(Pass, TOKEN_ID, OwnerMsk, 0);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status) <> 0) then
    begin
      Memo.Lines.Add('Open error' + CRLF + '--------------------');
      exit;
    end;
  Memo.Lines.Add(cOK);

  // wywoanie GetFeature w celu odczytania parametrw, TeleTokenu
  btGetFeatureClick(nil);
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btGetFeatureClick(Sender: TObject);
var
  n         : Byte;
  cFeature  : Cardinal;
  sTmp      : String;
begin
  Memo.Lines.Add(CRLF + 'GetFeature');

  // ptla odczytujca parametry TeleTokenu
  //---------------------------------------
  for n:=0 to (TVTTTI_TABLE_SIZE - 1) do
    begin
      // odczyt kolejnego parametru
      cFeature:=TVTTAPI_GetFeature(Pass, hCon, n);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<>0) then
        begin
          Memo.Lines.Add('GetFeature error' + CRLF + '--------------------');
          exit;
        end;

      // zapamitanie parametru TeleTokenu w tablicy
      TTInfo[n]:=cFeature;

      // dodanie parametru do linii tekstu
      sTmp:=sTmp + InfoText[n] + IntToStr(cFeature) + CRLF;
    end;
  //---------------------------------------
  // wywietlenie odczytanych danych o TeleTokenie
  Memo.Lines.Add(sTmp);
  Memo.Lines.Add(cOK);
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCloseClick(Sender: TObject);
begin
  // zamknicie TeleTokenu
  TVTTAPI_Close(Pass, hCon);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<>0) then
    begin
      Memo.Lines.Add('Close error' + CRLF + '--------------------');
      exit;
    end;

  // zwolnienie uchwytu na TeleToken
  hCon:=nil;

  Memo.Lines.Add('Close OK' + CRLF + '--------------------');
end;

//---------------------------------------------------------------------------
procedure TfrmRoot.btRAMWriteClick(Sender: TObject);
var
  Buff    : array [0..15] of byte;  // bufor na odczytane dane
  sText   : string;                 // linia tekstu
  n       : byte;                   // zmienna pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // wypenienie bufora przykadowymi wartociami
  TVTTAPI_Rand(Buff);

  // zapis bufora do pamici RAM
  TVTTAPI_RamWrite(Pass, hCon, 0, Buff);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<>0) then
    begin
      Memo.Lines.Add('RamWrite error' + CRLF + '--------------------');
      CloseSession();
      exit;
    end;

  // wywietlenie danych zapisanych do pamici
  Memo.Lines.Add(CRLF + 'RamWrite');
  n:=0;
  sText:='';
  while (n<16) do
    begin
      sText:=sText + ' ' + IntToHex(Buff[n],2);
      inc(n);
    end;

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btRAMReadClick(Sender: TObject);
var
  Buff    : array [0..15] of byte;  // bufor na odczytane dane
  sText   : string;                 // linia tekstu
  n       : byte;                   // zmienna pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // odczyt pamici RAM
  TVTTAPI_RamRead(Pass, hCon, 0, Buff);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<>0) then
    begin
      Memo.Lines.Add('RamRead error' + CRLF + '--------------------');
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  // wywietlenie odczytanych danych
  Memo.Lines.Add(CRLF + 'RamRead');
  n:=0;
  sText:='';
  while (n<16) do
    begin
      sText:=sText + ' ' + IntToHex(Buff[n],2);
      inc(n);
    end;

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btMemWriteClick(Sender: TObject);
var
  Buff    : array [0..15] of byte;  // bufor na odczytane dane
  sText   : string;                 // linia tekstu
  n,k     : byte;                   // zmienne pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zapisu do pamici TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'MemWrite');
  while (n<TTInfo[TVTTTI_NV_MEM_SIZE]) do
    begin
      // wypenienie bufora przykadowymi wartociami
      TVTTAPI_Rand(Buff);

      // zapis bufora do pamici
      TVTTAPI_MemWrite(Pass, hCon, n, Buff);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('MemWrite error' + CRLF + '--------------------');
          CloseSession();
          exit;
        end;

      // wywietlenie danych zapisanych do pamici
      k:=0;
      sText:='';
      while (k<16) do
        begin
          sText:=sText + ' ' + IntToHex(Buff[k],2);
          inc(k);
        end;
      Memo.Lines.Add(sText);

      // zwikszenie licznika sektorw
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btMemReadClick(Sender: TObject);
var
  Buff    : array [0..15] of byte;  // bufor na odczytane dane
  sText   : string;                 // linia tekstu
  n,k     : byte;                   // zmienne pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla odczytu z pamici TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'MemRead');
  while (n<TTInfo[TVTTTI_NV_MEM_SIZE]) do
    begin
      // odczyt z pamici
      TVTTAPI_MemRead(Pass, hCon, n, Buff);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('MemRead error' + CRLF + '--------------------');
          CloseSession();
          exit;
        end;

      // wywietlenie danych odczytanych z pamici
      k:=0;
      sText:='';
      while (k<16) do
        begin
          sText:=sText + ' ' + IntToHex(Buff[k],2);
          inc(k);
        end;
      Memo.Lines.Add(sText);

      // zwikszenie licznika sektorw
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btMemWProtectClick(Sender: TObject);
var
  Buff    : array [0..15] of byte;  // bufor na odczytane dane
  sText   : string;                 // linia tekstu
  n,k     : byte;                   // zmienne pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zapisu do pamici TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'MemWriteProtect');
  while (n<TTInfo[TVTTTI_PROTECT_MEM_SIZE]) do
    begin
      // wypenienie bufora przykadowymi wartociami
      TVTTAPI_Rand(Buff);

      // zapis bufora do pamici
      TVTTAPI_MemWriteProtect(Pass, hCon, n, Buff);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('MemWriteProtect error' + CRLF + '--------------------');
          CloseSession();                               // zamknicie sesji
          exit;
        end;

      // wywietlenie danych zapisanych do pamici
      k:=0;
      sText:='';
      while (k<16) do
        begin
          sText:=sText + ' ' + IntToHex(Buff[k],2);
          inc(k);
        end;
      Memo.Lines.Add(sText);

      // zwikszenie licznika sektorw
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btMemProtectClick(Sender: TObject);
var
  n       : byte;                   // zmienne pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zabezpieczajca zapis do sektorw TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'MemProtect');
  while (n<TTInfo[TVTTTI_PROTECT_MEM_SIZE]) do
    begin
      // zabezpieczenie zapisu do pamici sektora n
      TVTTAPI_MemProtect(Pass, hCon, n);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('MemProtect error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // zwikszenie licznika sektorw
      inc(n);
  end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterWriteClick(Sender: TObject);
var
  n       : byte;                   // zmienne pomocnicza
  sText   : string;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zapisu wartoci licznikw
  //---------------------------------------
  n:=0;
  sText:='';
  Memo.Lines.Add(CRLF + 'CounterWrite');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
      // zapis wartoci licznika
      TVTTAPI_CounterWrite(Pass, hCon, n, ($55 + n));

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('CounterWrite error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;
      sText:=sText + ' ' + IntToHex($55 + n,2);

      // zwikszenie wartoci licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterReadClick(Sender: TObject);
var
  n       : byte;                   // zmienne pomocnicza
  cnt     : cardinal;               // warto licznika
  sText   : string;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla odczytu wartoci licznikw
  //---------------------------------------
  n:=0;
  sText:='';
  Memo.Lines.Add(CRLF + 'CounterRead');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
      // odczyt wartoci licznikw
      cnt:=TVTTAPI_CounterRead(Pass, hCon, n);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('CounterRead error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;
      sText:=sText + ' ' + IntToHex(cnt,2);

      // zwikszenie wartoci licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterIncReadClick(Sender: TObject);
var
  n       : byte;                   // zmienne pomocnicza
  cnt     : cardinal;               // warto licznika
  sText   : string;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zwikszajca i odczytujca wartoci
  // licznikw
  //---------------------------------------
  n:=0;
  sText:='';
  Memo.Lines.Add(CRLF + 'CounterIncRead');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
    // zwikszenie i odczyt wartoci licznika
    cnt:=TVTTAPI_CounterIncRead(Pass, hCon, n);

    // obsuga bdw
    Status:=GetLastError();
    if (CheckErr(Status)<> 0) then
      begin
      Memo.Lines.Add('CounterIncRead error' + CRLF + '--------------------');
      CloseSession();                                   // zamknicie sesji
      exit;
      end;

    sText:=sText + ' ' + IntToHex(cnt,2);

    // zwikszenie wartoci licznika ptli
    inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterDecReadClick(Sender: TObject);
var
  n       : byte;                   // zmienne pomocnicza
  cnt     : cardinal;               // warto licznika
  sText   : string;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zmniejszajca i odczytujca wartoci
  // licznikw
  //---------------------------------------
  n:=0;
  sText:='';
  Memo.Lines.Add(CRLF + 'CounterDecRead');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
      // zmniejszenie i odczyt wartoci licznika
      cnt:=TVTTAPI_CounterDecRead(Pass, hCon, n);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('CounterDecRead error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      sText:=sText + ' ' + IntToHex(cnt,2);

      // zwikszenie wartoci licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterIncEncryptReadClick(Sender: TObject);
var
  Buff    : array [0..15] of byte;  // bufor na dane
  sText   : string;                 // linia tekstu
  n,k     : byte;                   // zmienne pomocnicza
  tTmp    : array [0..15] of byte;
  m       : byte;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zwikszajca i odczytujca wartoci
  // licznikw w postaci zaszyfrowanej
  //---------------------------------------
  n:=0;
  sText:='';
  Memo.Lines.Add(CRLF + 'CounterIncCryptRead');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
      // zwikszenie i odczyt zaszyfrowanej wartoci licznika
      TVTTAPI_CounterIncEncryptRead(Pass, hCon, n, n, Buff );

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('CounterIncEncryptRead error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // rozszyfrowanie i wywietlenie stanu licznika
      m:=0;
      while (m<16) do
        begin
          tTmp[m] := Buff[m];
          m:=m + 1;
        end;
      rijndaelDecrypt(PassTbl[n * 16], tTmp);               //odszyfrowanie odczytanego stanu licznika

      sText := IntToHex(tTmp[1], 2);
      sText := sText + IntToHex(tTmp[0], 2) + '| ';

      // wywietlenie zaszyfrowanych wartoci zwrconych przez funkcje
      k:=0;
      while (k<16) do
        begin
          sText:=sText + ' ' + IntToHex(Buff[k],2);
          inc(k);
        end;
      Memo.Lines.Add(sText);

      // zwikszenie wartoci licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterConfigWriteClick(Sender: TObject);
var
  n,                          // zmienna pomocnicza
  Config0, Config1 : byte;    // zmienne dla konfiguracji licznikw
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zapisujca konfiguracje licznikw
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'CounterConfigWrite');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
      // konfiguracja licznikw
      Config0:=TVTTCC0_WRITE or TVTTCC0_READ or TVTTCC0_INC_READ or TVTTCC0_DEC_READ or
                TVTTCC0_INC_ENCRYPT_READ or TVTTCC0_STOP_AT_ZERO or TVTTCC0_NVMEM_READ or
                TVTTCC0_NVMEM_WRITE;
      Config1:=TVTTCC1_PASS or TVTTCC1_CONFIG_ENABLED;

      // zapis konfiguracji
      TVTTAPI_CounterConfigWrite(Pass, hCon, n, Config0, Config1);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('CounterConfigWrite error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // zwikszenie wartoci licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterConfigSetClick(Sender: TObject);
var
  n,                          // zmienna pomocnicza
  Config0, Config1 : byte;    // zmienne dla konfiguracji licznikw
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla ustawiajca bity konfiguracji licznikw
  // (wszystkie wczone)
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'CounterConfigSet');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
      // ustawienie bitw konfiguracyjnych
      Config0:=TVTTCC0_WRITE or TVTTCC0_READ or TVTTCC0_INC_READ or TVTTCC0_DEC_READ or
                TVTTCC0_INC_ENCRYPT_READ or TVTTCC0_STOP_AT_ZERO or TVTTCC0_NVMEM_READ or
                TVTTCC0_NVMEM_WRITE;
      Config1:=TVTTCC1_PASS or TVTTCC1_CONFIG_ENABLED;

      // przesanie do TeleTokenu bitw konfiguracji do ustawienia
      TVTTAPI_CounterConfigSet(Pass, hCon, n, Config0, Config1);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('CounterConfigSet error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // zwikszenie wartoci licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btCounterConfigClearClick(Sender: TObject);
var
  n,                          // zmienna pomocnicza
  Config0, Config1 : byte;    // zmienne dla konfiguracji licznikw
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zerujca bity konfiguracji licznikw
  // (wszystkie wyczone)
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'CounterConfigClear');
  while (n<TTInfo[TVTTTI_COUNTERS]) do
    begin
    // bity konfiguracji do wyzerowania
    Config0:=TVTTCC0_WRITE or TVTTCC0_READ or TVTTCC0_INC_READ or TVTTCC0_DEC_READ or
              TVTTCC0_INC_ENCRYPT_READ or TVTTCC0_STOP_AT_ZERO or TVTTCC0_NVMEM_READ or
              TVTTCC0_NVMEM_WRITE;
    Config1:=TVTTCC1_PASS or TVTTCC1_CONFIG_ENABLED;

    // przesanie do TeleTokenu bitw konfiguracji do wyzerowania
    TVTTAPI_CounterConfigClear(Pass, hCon, n, Config0, Config1);

    // obsuga bdw
    Status:=GetLastError();
    if (CheckErr(Status)<> 0) then
      begin
      Memo.Lines.Add('CounterConfigClear error' + CRLF + '--------------------');
      CloseSession();                                   // zamknicie sesji
      exit;
      end;

    // zwikszenie wartoci licznika ptli
    inc(n);
    end;
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btPassWriteClick(Sender: TObject);
var
  n       : byte;                   // zmienne pomocnicza
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zapisujca hasa do TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'PassWrite');
  while (n<TTInfo[TVTTTI_PASS_TABLE_SIZE]) do
    begin
    // zapis hase do TeleTokenu
    TVTTAPI_PassWrite(Pass, hCon, n, PassTbl[n * 16]);

    // obsuga bdw
    Status:=GetLastError();
    if (CheckErr(Status)<> 0) then
      begin
      Memo.Lines.Add('PassWrite error' + CRLF + '--------------------');
      CloseSession();                                   // zamknicie sesji
      exit;
      end;

    // zwikszenie licznika ptli
    inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btPassConfigWriteClick(Sender: TObject);
var
  n,                          // zmienna pomocnicza
  Config0, Config1 : byte;    // zmienne dla konfiguracji
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zapisujca konfiguracje hase do TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'PassConfigWrite');
  while (n<TTInfo[TVTTTI_PASS_TABLE_SIZE]) do
    begin
      // konfiguracja bitw hase
      Config0:=TVTTPC0_TOKEN or TVTTPC0_PASS_WRITE or TVTTPC0_PASS_CONFIG or
                TVTTPC0_MEM_READ or TVTTPC0_MEM_WRITE or TVTTPC0_ENCRYPT or
                TVTTPC0_DECRYPT;
      Config1:=TVTTPC1_CNT_READ or TVTTPC1_CNT_WRITE or TVTTPC1_CNT_CONFIG or
                TVTTPC1_ENCRYPT_PASS or TVTTPC1_DECRYPT_PASS or TVTTPC1_CNT_ENCRYPT_PASS or
                TVTTPC1_WRITE_ENABLED or TVTTPC1_CONFIG_ENABLED;
      // przesanie do TeleTokenu bitw konfiguracji hase
      TVTTAPI_PassConfigWrite(Pass, hCon, n, Config0, Config1);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('PassConfigWrite error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // zwikszenie licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btPassConfigSetClick(Sender: TObject);
var
  n,                          // zmienna pomocnicza
  Config0, Config1 : byte;    // zmienne dla konfiguracji licznikw
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;
  
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'PassConfigSet');
  while (n<TTInfo[TVTTTI_PASS_TABLE_SIZE]) do
    begin
      // konfiguracja bitw hase
      Config0:=TVTTPC0_TOKEN or TVTTPC0_PASS_WRITE or TVTTPC0_PASS_CONFIG or
                TVTTPC0_MEM_READ or TVTTPC0_MEM_WRITE or TVTTPC0_ENCRYPT or
                TVTTPC0_DECRYPT;
      Config1:=TVTTPC1_CNT_READ or TVTTPC1_CNT_WRITE or TVTTPC1_CNT_CONFIG or
                TVTTPC1_ENCRYPT_PASS or TVTTPC1_DECRYPT_PASS or TVTTPC1_CNT_ENCRYPT_PASS or
                TVTTPC1_WRITE_ENABLED or TVTTPC1_CONFIG_ENABLED;

      // przesanie do TeleTokenu bitw konfiguracji hase do ustawienia
      TVTTAPI_PassConfigSet(Pass, hCon, n, Config0, Config1);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('PassConfigSet error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // zwikszenie licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.PassConfigClearClick(Sender: TObject);
var
  n,                          // zmienna pomocnicza
  Config0, Config1 : byte;    // zmienne dla konfiguracji licznikw
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  // ptla zerujca bity konfiguracji hase TeleTokenu
  //---------------------------------------
  n:=0;
  Memo.Lines.Add(CRLF + 'PassConfigClear');
  while (n<TTInfo[TVTTTI_PASS_TABLE_SIZE]) do
    begin
      // konfiguracja bitw hase
      Config0:=TVTTPC0_TOKEN or TVTTPC0_PASS_WRITE or TVTTPC0_PASS_CONFIG or 
                TVTTPC0_MEM_READ or TVTTPC0_MEM_WRITE or TVTTPC0_ENCRYPT or
                TVTTPC0_DECRYPT;
      Config1:=TVTTPC1_CNT_READ or TVTTPC1_CNT_WRITE or TVTTPC1_CNT_CONFIG or 
                TVTTPC1_ENCRYPT_PASS or TVTTPC1_DECRYPT_PASS or TVTTPC1_CNT_ENCRYPT_PASS or
                TVTTPC1_WRITE_ENABLED or TVTTPC1_CONFIG_ENABLED;

      // przesanie do TeleTokenu bitw konfiguracji hase do wyzerowania
      TVTTAPI_PassConfigClear(Pass, hCon, n, Config0, Config1);

      // obsuga bdw
      Status:=GetLastError();
      if (CheckErr(Status)<> 0) then
        begin
          Memo.Lines.Add('PassConfigClear error' + CRLF + '--------------------');
          CloseSession();                                   // zamknicie sesji
          exit;
        end;

      // zwikszenie licznika ptli
      inc(n);
    end;
  //---------------------------------------

  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btEncryptClick(Sender: TObject);
var
  n,k     : byte;                   // zmienne pomocnicza
  sText   : string;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  //wywietlenie informacji o danych wejciowych
  Memo.Lines.Add(CRLF + 'Encrypt');
  sText:='Test crypt....';
  for n:=1 to length(sText) do
    CryptBuff[n-1]:=byte(sText[n]);
  sText:=CRLF + 'Input ASCII:' + CRLF + CRLF + sText + CRLF;
  Memo.Lines.Add(sText);
  Memo.Lines.Add('Input HEX:' + CRLF);
  k:=0;
  sText:='';
  while (k<16) do
    begin
      sText:=sText + ' ' + IntToHex(CryptBuff[k], 2);
      inc(k);
    end;
  Memo.Lines.Add(sText);

  // szyfrowanie bloku TeleTokenem przy uyciu pierwszego hasa
  TVTTAPI_Encrypt(Pass, hCon, 1, CryptBuff);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<> 0) then
    begin
      Memo.Lines.Add('Encrypt error' + CRLF + '--------------------');
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  // wywietlenie zaszyfrowanego bloku
  Memo.Lines.Add(CRLF + 'Output HEX:' + CRLF);
  k:=0;
  sText:='';
  while (k<16) do
    begin
      sText:=sText + ' ' + IntToHex(CryptBuff[k], 2);
      inc(k);
    end;
  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zamknicie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btDecryptClick(Sender: TObject);
var
  n,k     : byte;                   // zmienne pomocnicza
  sText   : string;
begin
  // otwarcie sesji
  if (OpenSession()<>0) then exit;

  //wywietlenie informacji o danych wejciowych
  k:=0;
  sText:='';
  Memo.Lines.Add(CRLF + 'Decrypt' + CRLF);
  Memo.Lines.Add('Input HEX:' + CRLF);
  while (k<16) do
    begin
      sText:=sText + ' ' + IntToHex(CryptBuff[k],2);
      inc(k);
    end;
  Memo.Lines.Add(sText);

  // deszyfrowanie bloku przy uyciu pierwszego hasa
  TVTTAPI_Decrypt(Pass, hCon, 1, CryptBuff);

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<> 0) then
    begin
      Memo.Lines.Add('Decrypt error' + CRLF + '--------------------');
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  // wywietlenie danych wyjciowych
  sText:='                ';
  for n:=0 to length(sText) do
    sText[n+1]:=char(CryptBuff[n]);
  sText:=CRLF + 'Output ASCII:' + CRLF + CRLF + sText + CRLF;
  Memo.Lines.Add(sText);

  Memo.Lines.Add(CRLF + 'Output HEX:' + CRLF);
  k:=0;
  sText:='';
  while (k<16) do
    begin
      sText:=sText + ' ' + IntToHex(CryptBuff[k], 2);
      inc(k);
    end;

  Memo.Lines.Add(sText);
  Memo.Lines.Add(cOK);

  // zakoczenie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btEncryptBlockClick(Sender: TObject);
var
  sErr     : string;
  inp, outp: file of byte; // uchwyty na pliki wejcia/wyjcia
  FName    : string;       // nazwa pliku do odczytu
  FSize    : Cardinal;     // rozmiar pliku do zaszyfrowania
  Buff     : pointer;      // bufor na dane wejciowe
  Crypt    : pointer;      // bufor na dane wyjciowe
  OutSize  : Cardinal;     // rozmiar w bajtach bloku zaszyfrowanego
begin
  sErr:='EncryptBlock error' + CRLF + '--------------------';

  Memo.Lines.Add(CRLF + 'EncryptBlock');

  // otwarcie okna dialogu
  if (not OpenDlg.Execute) then
    begin
      Memo.Lines.Add(sErr);
      exit;
    end;

  // otwarcie pliku rda
  AssignFile(inp,OpenDlg.FileName);
  {$I-}
  Reset(inp);
  {$I+}
  if IOResult<>0 then
   begin
      Memo.Lines.Add(sErr);
      exit;
   end;

  // otwarcie pliku docelowego (zakodowanego)
  FName:=OpenDlg.FileName + '.crp';
  AssignFile(outp,FName);
  {$I-}
  ReWrite(outp);
  {$I+}
  if IOResult<>0 then
   begin
      Memo.Lines.Add(sErr);
      CloseFile(inp);
      exit;
   end;

  // otwarcie sesji
  if (OpenSession()<>0) then
    begin
      CloseFile(inp);
      CloseFile(outp);
      exit;
    end;

  // alokowanie bufora pamici na dane
  FSize:=FileSize(inp);
  Buff:=Pointer(GlobalAlloc(GPTR,FSize));
  if Buff=nil then
    begin
      Memo.Lines.Add(sErr);
      CloseFile(inp);
      CloseFile(outp);
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  // odczyt pliku do zaszyfrowania
  BlockRead(inp, PByte(Buff)^, FSize);

  // szyfrowanie bloku danych TeleTokenem
  Crypt:=TVTTAPI_EncryptBlock(Pass, hCon, 1, Buff, FSize, OutSize);

  // zwolnienie pamici danych do zaszyfrowania
  GlobalFree(Cardinal(Buff));

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<> 0) then
    Memo.Lines.Add(sErr)
  else
    // zapis zaszyfrowanych danych do pliku
    BlockWrite(outp, PByte(Crypt)^, OutSize);

  // zamknicie plikw
  CloseFile(inp);
  CloseFile(outp);

  // zwolnienie pamici danych zaszyfrowanych
  TVTTAPI_FreeBlock(Crypt, OutSize);
  if (CheckErr(Status)<> 0) then
    begin
      Memo.Lines.Add(sErr);
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  Memo.Lines.Add(cOK);

  // zakoczenie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btDecryptBlockClick(Sender: TObject);
var
  sErr     : string;
  inp, outp: file of byte; // uchwyty na pliki wejcia/wyjcia
  FName    : string;       // nazwa zaszyfrowanego pliku do odczytu
  Buff     : Pointer;      // bufor na zaszyfrowane dane wejciowe
  DeCrypt  : Pointer;      // bufor na dane wyjciowe
  OutSize  : Cardinal;     // rozmiar w bajtach bloku zdeszyfrowanego
  FSize    : integer;      // rozmiar pliku do zdeszyfrowania
begin
  sErr:='DecryptBlock error' + CRLF + '--------------------';

  Memo.Lines.Add(CRLF + 'DecryptBlock');

  // otwarcie okna dialogu
  OpenDlg.Filter:='Encrypt files (*.crp) or *.CRP';
  if (not OpenDlg.Execute) then
    begin
      Memo.Lines.Add(sErr);
      exit;
    end;

  // otwarcie pliku rda
  AssignFile(inp,OpenDlg.FileName);
  {$I-}
  Reset(inp);
  {$I+}
  if IOResult<>0 then
    begin
      Memo.Lines.Add(sErr);
      exit;
    end;

  // otwarcie pliku docelowego (zakodowanego)
  FName:=OpenDlg.FileName+'.dcrp';
  AssignFile(outp,FName);
  {$I-}
  Rewrite(outp);
  {$I+}
  if IOResult<>0 then
    begin
      Memo.Lines.Add(sErr);
      CloseFile(inp);
      exit;
    end;

  // otwarcie sesji
  if (OpenSession()<>0) then
    begin
      CloseFile(inp);
      CloseFile(outp);
      exit;
    end;

  // alokowanie bufora pamici na dane
  FSize:=FileSize(inp);
  Buff:=Pointer(GlobalAlloc(GPTR,FSize));
  if Buff=nil then
    begin
      Memo.Lines.Add(sErr);
      CloseFile(inp);
      CloseFile(outp);
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  // odczyt pliku do odszyfrowania
  BlockRead(inp,PByte(Buff)^,FSize);

  //odszyfrowanie pliku
  Decrypt:=TVTTAPI_DecryptBlock(Pass, hCon, 1, Buff, OutSize);
  GlobalFree(Cardinal(Buff));

  // obsuga bdw
  Status:=GetLastError();
  if (CheckErr(Status)<> 0) then
    Memo.Lines.Add(sErr)
  else
    // zapis zaszyfrowanych danych do pliku
    BlockWrite(outp,PByte(Decrypt)^, OutSize);

  // zamknicie plikw
  CloseFile(inp);
  CloseFile(outp);

  // zwolnienie pamici danych rozszyfrowanych
  TVTTAPI_FreeBlock(Decrypt, OutSize);
  if (CheckErr(Status)<> 0) then
    begin
      Memo.Lines.Add(sErr);
      CloseSession();                                   // zamknicie sesji
      exit;
    end;

  Memo.Lines.Add(cOK);

  // zakoczenie sesji
  CloseSession();
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.btLosujClick(Sender: TObject);
var
  tRandom  : array [0..15] of byte;
  sText    : string;
  sLine    : string;
  n        : byte;
begin
  // generowanie liczb pseudolosowych
  TVTTAPI_Rand(tRandom);
  for n:=0 to 15 do
    begin
      sText:=IntToHex(tRandom[n], 2);
      sLine:=sLine + sText + ' ';
    end;

  // wywietlenie wygenerowanych liczb
  Memo.Lines.Add(sLine);
end;
//---------------------------------------------------------------------------
procedure TfrmRoot.ApplicationEventsMessage(var Msg: tagMSG;
  var Handled: Boolean);
begin
  //dodanie zdarzenia losowego do generatora liczb pseudolosowych
  TVTTAPI_Randomize(Msg);
end;
//---------------------------------------------------------------------------
initialization
  OwnerMsk:=0;
  hCon:=nil;
end.
