unit TeleTokenAPI;
interface
uses
  SysUtils,
  Windows,
  Messages,
  Rijndael,
  TeleToken,
  ZeroMem,
  Dialogs;

const
    ERROR_NOT_FOUND = 1168;
    ERROR_SET_NOT_FOUND = 1170;
    TVTTAPI_Int_Name = 'Software\\App Data\\Seed';

type  TVTTAPI_INT_CON = packed record
  Rand          : array [0..15] of byte;
  RandPC        : array [0..15] of byte;
  RandT         : array [0..15] of byte;
  Key           : array [0..15] of byte;
  hToken        : cardinal;
  OutKey        : cardinal;
  InKey         : cardinal;
  TokenInfo     : array [0..255] of cardinal;
  TokenInfoSize : cardinal;
end;

var
  TVTTAPI_Int_LastUpdate  : cardinal;

  procedure TVTTAPI_Int_CodeCon(var Pass: array of Byte; hCon : Pointer);
  procedure TVTTAPI_Int_DecodeCon(var Pass : array of Byte; hCon : Pointer);
  procedure TVTTAPI_Int_SetLastError(Res : Cardinal);
  function  TVTTAPI_Int_CheckError(hCon : Pointer; Res : cardinal; InKey : cardinal; InDataA, InDataB : array of byte) : byte;
  procedure TVTTAPI_Initialize();
  procedure TVTTAPI_Randomize(Msg : MSG);
  procedure TVTTAPI_Rand(var Random : array of byte);
  function  TVTTAPI_Open(var Pass : array of byte;  OwnerID : Cardinal; OwnerMsk : byte; Skip : byte) : Pointer;
  procedure TVTTAPI_Close(Pass : array of byte; hCon : Pointer);
  function  TVTTAPI_GetFeature(Pass : array of byte; hCon : Pointer; FNr : Cardinal):Cardinal;
  function  TVTTAPI_StartSession(Pass : array of byte; hCon : Pointer; PCPassNr: byte):byte;
  procedure TVTTAPI_EndSession(Pass : array of byte; hCon : Pointer);
  procedure TVTTAPI_CalcPass(var Pass : array of byte; hCon : Pointer; var PassPC : array of byte; var PassT : array of byte);
  procedure TVTTAPI_MemRead(Pass : array of byte; hCon : Pointer; SNr : byte; var Buff : array of byte);
  procedure TVTTAPI_RamRead(Pass : array of byte; hCon : Pointer; SNr : byte; var Buff : array of byte);
  procedure TVTTAPI_RamWrite(Pass : array of byte; hCon : Pointer; SNr : byte; Buff : array of byte);
  procedure TVTTAPI_MemWrite(Pass : array of byte; hCon : Pointer; SNr : byte; Buff : array of byte);
  procedure TVTTAPI_MemWriteProtect(Pass : array of byte; hCon : Pointer; SNr : byte; Buff : array of byte);
  procedure TVTTAPI_MemProtect(Pass : array of byte; hCon : Pointer; SNr : byte);
  procedure TVTTAPI_Encrypt(Pass : array of byte; hCon : Pointer; PNr : byte; var Buff : array of byte);
  procedure TVTTAPI_Decrypt(Pass : array of byte; hCon : Pointer; PNr : byte; var Buff: array of byte);
  function  TVTTAPI_CounterIncRead(Pass : array of byte; hCon : Pointer; CNr : byte) : Cardinal;
  procedure TVTTAPI_CounterWrite(Pass : array of byte; hCon : Pointer; CNr : byte; Value : Cardinal);
  function  TVTTAPI_CounterRead(Pass : array of byte; hCon : Pointer; CNr : byte) : Cardinal;
  function  TVTTAPI_CounterDecRead(Pass : array of byte; hCon : Pointer; CNr : byte) : Cardinal;
  procedure TVTTAPI_CounterIncEncryptRead(Pass : array of byte; hCon : Pointer; CNr, PNr : byte; var Buff : array of byte);
  procedure TVTTAPI_CounterConfigWrite(Pass : array of byte; hCon : Pointer; CNr : byte; Config0,Config1 : byte);
  procedure TVTTAPI_CounterConfigSet(Pass : array of byte; hCon : Pointer; CNr : byte; Config0,Config1 : byte);
  procedure TVTTAPI_CounterConfigClear(Pass : array of byte; hCon : Pointer; CNr : byte; Config0,Config1 : byte);
  procedure TVTTAPI_PassWrite(Pass : array of byte; hCon : Pointer; PNr : byte; var Buff : array of byte);
  procedure TVTTAPI_PassConfigWrite(Pass : array of byte; hCon : Pointer; PNr : byte; Config0, Config1 : byte);
  procedure TVTTAPI_PassConfigSet(Pass : array of byte; hCon : Pointer; PNr : byte; Config0, Config1 : byte);
  procedure TVTTAPI_PassConfigClear(Pass : array of byte; hCon : Pointer; PNr : byte; Config0, Config1 : byte);
  function  TVTTAPI_EncryptBlock(Pass : array of byte; hCon : Pointer; PNr : byte; Buff : Pointer; Size : Cardinal; var OutSize : Cardinal) : Pointer;
  function  TVTTAPI_DecryptBlock(Pass : array of byte; hCon : Pointer; PNr : byte; Buff : Pointer; var OutSize : Cardinal) : Pointer;
  procedure TVTTAPI_FreeBlock(Buff : Pointer; Size : Cardinal);
implementation

{*****************************************************************************
Funkcja wewntrzna - szyfruje strukture opisujc poczenie
We:
    Adres na klucz szyfrowania
    Adres na strukture do zaszyfrowania
Wy:
*****************************************************************************}
procedure TVTTAPI_Int_CodeCon(var Pass: array of Byte; hCon : Pointer);
var
  n,i : Cardinal;
  Con : ^TVTTAPI_INT_CON;
begin
  Con:=hCon;

  TVTT_Rand((PByte(Con))^);

  n:=16;
  while (n<(((sizeof(TVTTAPI_INT_CON)+15) div 16)*16)) do
    begin
      i:=0;
      while (i<16) do
        begin
          (PByte(Cardinal(Con) + (n+i)))^:=(PByte(Cardinal(Con) + n+i))^ xor (PByte(Cardinal(Con) + (n+i-16)))^;
          inc(i);
        end;

      rijndaelEncrypt(Pass,PByte(Cardinal(Con)+n)^);
      inc(n,16);
    end;
end;

{*****************************************************************************
Funkcja wewntrzna - rozszyfruje strukture opisujc poczenie
We:
    Adres na klucz szyfrowania
    Adres na strukture do zaszyfrowania
Wy:
*****************************************************************************}
procedure TVTTAPI_Int_DecodeCon(var Pass : array of Byte; hCon : Pointer);
var
  n,i : Cardinal;
  Con : ^TVTTAPI_INT_CON;
begin

  Con:=hCon;

  n:=(((sizeof(TVTTAPI_INT_CON)+15) div 16)*16)-16;

  while (n>15) do
    begin
      rijndaelDecrypt(Pass,PByte(Cardinal(Con)+n)^);
      i:=0;
      while(i<16) do
        begin
          (PByte(Cardinal(Con) + (n+i)))^:=(PByte(Cardinal(Con) + n+i))^ xor (PByte(Cardinal(Con) + (n+i-16)))^;
          inc(i);
        end;

      n:=n-16;
    end;
end;

{*****************************************************************************
Funkcja wewntrzna - na podstawie zwrconego kodu bdu ustawia "LastError"
We:
    kod bdu
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Int_SetLastError(Res : Cardinal);
begin
  case (Res and $FF00) of
    TVTTS_TOO_MANY_OPEN_TOKENS:
      begin
        SetLastError(ERROR_TOO_MANY_OPEN_FILES);
        exit;
      end;

    TVTTS_USB_DOES_NOT_EXIST:
      begin
        SetLastError(ERROR_UNKNOWN_PORT);
        exit;
      end;

    TVTTS_TOKEN_NOT_FOUND:
      begin
        SetLastError(ERROR_NOT_FOUND);
        exit;
      end;

    TVTTS_TIME_OUT:
      begin
        SetLastError(ERROR_TIMEOUT);
        exit;
      end;

    TVTTS_EXPIRED:
      begin
        SetLastError(ERROR_ACCOUNT_EXPIRED);
        exit;
      end;

    TVTTS_ALREADY_OPENED:
      begin
        SetLastError(ERROR_ALREADY_INITIALIZED);
        exit;
      end;

    TVTTS_UNKNOWN_ERROR,
    TVTTS_NULL_PTR,
    TVTTS_BAD_PARAM,
    TVTTS_INVALID_HANDLE:
      begin
        SetLastError(ERROR_UNEXP_NET_ERR);
        exit;
      end;
    end;

  case (Res and $FF) of
    TVTTS_NO_OPEN:
      begin
        SetLastError(ERROR_ACCOUNT_EXPIRED);
        exit;
      end;

    TVTTS_EXPIRED:
      begin
        SetLastError(ERROR_TIMEOUT);
        exit;
      end;

    TVTTS_DAMAGED:
      begin
        SetLastError(ERROR_GEN_FAILURE);
        exit;
      end;

    TVTTS_NOT_SUPPORTED:
      begin
        SetLastError(ERROR_NOT_SUPPORTED);
        exit;
      end;

    TVTTS_OUT_OF_RANGE:
      begin
        SetLastError(ERROR_INVALID_BLOCK);
        exit;
      end;

    TVTTS_PROTECTION_NOT_SUPPORTED:
      begin
        SetLastError(ERROR_SET_NOT_FOUND);
        exit;
      end;

    TVTTS_PASS_NOT_PERMITTED:
      begin
        SetLastError(ERROR_INVALID_PASSWORD);
        exit;
      end;

    TVTTS_NOT_PERMITTED:
      begin
        SetLastError(ERROR_ACCESS_DENIED);
        exit;
      end;

    TVTTS_ACCESS_EXPIRED:
      begin
        SetLastError(ERROR_COUNTER_TIMEOUT);
        exit;
      end;
  end;

  if (Res<>0) then
      SetLastError(ERROR_UNEXP_NET_ERR)
  else
      SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja wewntrzna - testuje poprawno komunikacji
We:
    kod bdu
Wy:
	0 - bez bedw / <>0 - wystapi bd
    ustawiony "LastError"
*****************************************************************************}

function TVTTAPI_Int_CheckError(hCon : Pointer; Res : cardinal; InKey : cardinal; InDataA, InDataB : array of byte) : byte;
var
  Con : ^TVTTAPI_INT_CON;
begin
  Con:=hCon;

  if Res>0 then
    begin
      TVTTAPI_Int_SetLastError(Res);
      result:=1;
      exit;
    end;

  if InKey=0 then
    begin
      if TVTT_CheckAuth(InDataA,InDataB)>0 then
        SetLastError(ERROR_CRC)
      else
        if InDataB[TVTTD_STATUS]>0 then
          TVTTAPI_Int_SetLastError(InDataB[TVTTD_STATUS])
        else
          SetLastError(ERROR_UNEXP_NET_ERR);
      result:=1;
      exit;
    end
  else
    begin
      if (Con.InKey and $FF000000)>0 then
        Con.InKey:=InKey
      else
        if (InKey<>Con.InKey) then
          begin
            SetLastError(ERROR_UNEXP_NET_ERR);
            result:=1;
            exit;
          end;
      Con.InKey:=((Con.InKey+1) and $FFFFFF);
    end;

  if (TVTT_CheckAuth(InDataA,InDataB)>0) then
    begin
      SetLastError(ERROR_CRC);
      result:=1;
      exit;
    end;

  if (InDataB[TVTTD_STATUS]>0) then
    begin
      TVTTAPI_Int_SetLastError(InDataB[TVTTD_STATUS]);
      result:=1;
      exit;
    end;

  result:=0;
end;

{*****************************************************************************
Funkcja inicjalizuje bibliotek
We:
Wy:
*****************************************************************************}
procedure TVTTAPI_Initialize();
var
  InitTab     : array [0..TVTT_RAND_INIT_TBL_SIZE - 1] of byte;
  hKeyr       : HKEY;
  Res         : Cardinal;
  l           : Longint;
  n,i         : Cardinal;
begin
  n:=0;
  while (n<TVTT_RAND_INIT_TBL_SIZE) do
    begin
      InitTab[n]:=Byte(GetTickCount());
      inc(n);
    end;

  l:=RegCreateKeyEx(HKEY_CURRENT_USER,PChar(TVTTAPI_Int_Name),0,nil,
                    REG_OPTION_NON_VOLATILE,(KEY_QUERY_VALUE or KEY_SET_VALUE),
                    nil,hKeyr,@Res);

  if (l = ERROR_SUCCESS) and (Res = REG_OPENED_EXISTING_KEY) then
      begin
        n:=TVTT_RAND_INIT_TBL_SIZE;
        i:=REG_BINARY;
        RegQueryValueEx(hKeyr,nil,nil,@i,@InitTab,@n);
      end;

  TVTT_RandInit(InitTab);

  TVTT_RandGetInit(InitTab);
  RegSetValueEx(hKeyr,nil,0,REG_BINARY,@InitTab,TVTT_RAND_INIT_TBL_SIZE);

  if (l = ERROR_SUCCESS) then RegCloseKey(hKeyr);

  ZeroMemory(InitTab,TVTT_RAND_INIT_TBL_SIZE);
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja dodaje zdarzenia do generatora liczb pseudolosowych
We:
    Adres na wiadomo systemow
Wy:
*****************************************************************************}
procedure TVTTAPI_Randomize(Msg : MSG);
var
  InitTab   : array [0..TVTT_RAND_INIT_TBL_SIZE -1] of byte;
  n         : byte;
  Time      : Cardinal;
  l         : longint;
  hKeyr     : HKEY;
  Res       : Cardinal;
begin
  n:=$FF;

  case (Msg.message) of
    WM_KEYUP:
      n:=0;

    WM_KEYDOWN:
      n:=1;

    WM_MOUSEMOVE:
      n:=2;

    WM_LBUTTONDOWN:
      n:=3;

    WM_LBUTTONUP:
      n:=4;

    WM_MBUTTONDOWN:
      n:=5;

    WM_MBUTTONUP:
      n:=6;

    WM_RBUTTONDOWN:
      n:=7;

    WM_RBUTTONUP:
      n:=8;
  end;

  if ((n*3)<31) then
    begin
      TVTT_Randomize((n*3)+0,Msg.time);
      TVTT_Randomize((n*3)+1,(Cardinal(Msg.wParam)));
      TVTT_Randomize((n*3)+2,(Cardinal(Msg.lParam)));
    end;

  Time:=GetTickCount();
  if (TVTTAPI_Int_LastUpdate and $80000000)>0 then
    begin
      if (Time and $80000000)>0 then
        begin
          if ((TVTTAPI_Int_LastUpdate+10000)<TVTTAPI_Int_LastUpdate) then exit;
          if (Time<(TVTTAPI_Int_LastUpdate+10000)) then exit;
        end;
    end
  else
    if (Time<(TVTTAPI_Int_LastUpdate+10000)) then exit;

  TVTTAPI_Int_LastUpdate:=Time;

  l:=RegCreateKeyEx(HKEY_CURRENT_USER,TVTTAPI_Int_Name,0,nil,REG_OPTION_NON_VOLATILE,
                    KEY_QUERY_VALUE or KEY_SET_VALUE,nil,hKeyr,@Res);
  if ((l = ERROR_SUCCESS) and (Res = REG_OPENED_EXISTING_KEY)) then
      begin
        TVTT_RandGetInit(InitTab);
        RegSetValueEx(hKeyr,nil,0,REG_BINARY,@InitTab,TVTT_RAND_INIT_TBL_SIZE);
      end;
  if (l = ERROR_SUCCESS) then RegCloseKey(hKeyr);

  ZeroMemory(InitTab,TVTT_RAND_INIT_TBL_SIZE);
end;

{*****************************************************************************
Funkcja wypenia podany blok wartoci pseudolosow
We:
    Adres na tablice do wypenienia
Wy:
*****************************************************************************}
procedure TVTTAPI_Rand(var Random : array of byte);
begin
  if @Random = nil then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;
  TVTT_Rand(Random);
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja znajduje Token i otwiera poczenie z Tokenem
We:
    Adres na 16-bajtowy klucz do poczenia
    ID odbiorcy tokenu
    Maska odbiorcy tokenu
    Ilo Tokenw do pominicia
Wy:
    uchwyt na poczenie z tokenem lub NULL jeli bd
    ustawiony "LastError"
*****************************************************************************}
function  TVTTAPI_Open(var Pass : array of byte;  OwnerID : Cardinal; OwnerMsk : byte;
                     Skip : byte) : Pointer;
var
  Res : Cardinal;
  Con : ^TVTTAPI_INT_CON;
begin
  if @Pass=nil then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=nil;
      exit;
    end;

  Con:=Pointer(GlobalAlloc(GPTR,((sizeof(TVTTAPI_INT_CON)+15) div 16)*16));
  if Con=nil then
    begin
      result:=nil;
      exit;
    end;

  Con.TokenInfoSize:=256;
  Res:=TVTT_FindToken(@(Con.hToken),5*10,OwnerID,OwnerMsk,Skip,
                       @(Con.TokenInfoSize),@Con.TokenInfo);
  if Res>0 then
    begin
      TVTTAPI_Int_SetLastError(Res);
      ZeroMemory(PByte(Con)^,((sizeof(TVTTAPI_INT_CON)+15) div 16)*16);
      GlobalFree(Cardinal(Con));
      result:=nil;
      exit;
    end;
  TVTTAPI_Int_CodeCon(Pass,Con);
  SetLastError(ERROR_SUCCESS);

  result:=Con;
end;

{*****************************************************************************
Funkcja zamyka poczenie z Tokenem i zwalnia zasoby
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Close(Pass : array of byte; hCon : Pointer);
var
  Res : Cardinal;
  Con : ^TVTTAPI_INT_CON;
begin
  Con:=hCon;

  if((@Pass = nil) or (hCon = nil)) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  Res:=TVTT_FreeToken(Con.hToken);
  if (Res>0) then
    begin
      TVTTAPI_Int_CodeCon(Pass,Con);
      TVTTAPI_Int_SetLastError(Res);
      exit;
    end;

  ZeroMemory(PByte(Con)^,((sizeof(TVTTAPI_INT_CON)+15) div 16)*16);
  GlobalFree(Cardinal(hCon));
  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zwraca warto opisujc konkretn waciwo Tokenu
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer waciwoci Tokenu
Wy:
    waciwo Tokenu
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_GetFeature(Pass : array of byte; hCon : Pointer; FNr : Cardinal):Cardinal;
var
  Res : Cardinal;
  Con : ^TVTTAPI_INT_CON;
begin
  Con:=hCon;

  if((@Pass = nil) or (hCon = nil)) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=0;
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  if (FNr>=Con.TokenInfoSize) then
    Res:=0
  else
    Res:=Con.TokenInfo[FNr];

  TVTTAPI_Int_CodeCon(Pass,Con);

  SetLastError(ERROR_SUCCESS);
  Result:=Res;
end;

{*****************************************************************************
Funkcja rozpoczyna sesj z tokenem
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa proponowanego przez PC
Wy:
    Numer hasa zaproponowanego przez Token
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_StartSession(Pass : array of byte; hCon : Pointer; PCPassNr: byte):byte;
var
  Con     : ^TVTTAPI_INT_CON;
  Res     : Cardinal;
  TPassNr : byte;
  OutData : array [0..15] of byte;
  InData  : array [0..15] of byte;
  InKey   : Cardinal;
begin
  Con:=hCon;

  if((@Pass = nil) or (hCon = nil)) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=0;
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  Res:=TVTT_StartSession(Con.hToken,1);
  if (Res>0) then
    begin
      TVTTAPI_Int_CodeCon(Pass,Con);
      TVTTAPI_Int_SetLastError(Res);
      result:=0;
      exit;
    end;

  ZeroMemory(Con.RandPC,16);
  ZeroMemory(OutData,16);
  ZeroMemory(Con.RandT,16);
  ZeroMemory(InData,16);

  Con.OutKey:=0;
  Con.InKey:=$FFFFFFFF;
  TVTT_Rand(Con.RandPC);
  OutData[TVTTD_CMD]:=TVTTC_OPEN;
  OutData[TVTTD_PARAM1]:=PCPassNr;

  TVTT_CalcAuth(Con.RandPC,OutData);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@Con.RandPC,@OutData,@InKey,@Con.RandT,@InData);
  if (Res>0) then
    begin
      InKey:=0;
      ZeroMemory(Con.RandPC,16);
      ZeroMemory(OutData,16);
      ZeroMemory(Con.RandT,16);
      ZeroMemory(InData,16);
      TVTT_EndSession(Con.hToken);
      TVTTAPI_Int_CodeCon(Pass,Con);
      TVTTAPI_Int_SetLastError(Res);
      result:=0;
      exit;
    end;

  if InKey>0 then
    begin
      InKey:=0;
      ZeroMemory(Con.RandPC,16);
      ZeroMemory(OutData,16);
      ZeroMemory(Con.RandT,16);
      ZeroMemory(InData,16);
      TVTT_EndSession(Con.hToken);
      TVTTAPI_Int_CodeCon(Pass,Con);
      SetLastError(ERROR_UNEXP_NET_ERR);
      result:=0;
      exit;
    end;

  if TVTT_CheckAuth(Con.RandT,InData)>0 then
    begin
      InKey:=0;
      ZeroMemory(Con.RandPC,16);
      ZeroMemory(OutData,16);
      ZeroMemory(Con.RandT,16);
      ZeroMemory(InData,16);
      TVTT_EndSession(Con.hToken);
      TVTTAPI_Int_CodeCon(Pass,Con);
      SetLastError(ERROR_CRC);
      result:=0;
      exit;
    end;

  if InData[TVTTD_STATUS]>0 then
    begin
      TVTT_EndSession(Con.hToken);
      TVTTAPI_Int_CodeCon(Pass,Con);
      TVTTAPI_Int_SetLastError(InData[TVTTD_STATUS]);
      InKey:=0;
      ZeroMemory(Con.RandPC,16);
      ZeroMemory(OutData,16);
      ZeroMemory(Con.RandT,16);
      ZeroMemory(InData,16);
      result:=0;
      exit;
    end;

  TPassNr:=InData[TVTTD_PARAM1];

  ZeroMemory(OutData,16);
  ZeroMemory(InData,16);
  InKey:=0;

  TVTT_Rand(OutData);
  Con.OutKey:=OutData[0];
  Con.OutKey:=Con.OutKey shl 8;
  Con.OutKey:=Con.OutKey or OutData[1];
  Con.OutKey:=Con.OutKey shl 8;
  Con.OutKey:=Con.OutKey or OutData[2];
  Con.OutKey:=Con.OutKey shl 8;
  ZeroMemory(OutData,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  SetLastError(ERROR_SUCCESS);
  result:=TPassNr;
end;

{*****************************************************************************
Funkcja koczy sesj z tokenem
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_EndSession(Pass : array of byte; hCon : Pointer);
var
  Con     : ^TVTTAPI_INT_CON;
  Res     : Cardinal;
  Res2    : Cardinal;
  OutDataA: array [0..15] of byte;
  OutDataB: array [0..15] of byte;
  InDataA : array [0..15] of byte;
  InDataB : array [0..15] of byte;
  InKey   : Cardinal;
begin
  Con:=hCon;

  if((@Pass = nil) or (hCon = nil)) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  inc(Con.OutKey);
  OutDataB[TVTTD_CMD]:=TVTTC_CLOSE;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  if (Res>0) then TVTTAPI_Int_SetLastError(Res);

  Res2:=TVTT_EndSession(Con.hToken);
  if (Res=0) and (Res2>0) then TVTTAPI_Int_SetLastError(Res2);

  Con.OutKey:=0;
  ZeroMemory(Con.Key,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);
  InKey:=0;

  if (Res = 0) and (Res2 = 0) then
    SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja wylicza haso sesji na podstawie zaproponowanych hase
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Haso proponowane przez PC
    Haso proponowane przez T
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CalcPass(var Pass : array of byte; hCon : Pointer; var PassPC : array of byte; var PassT : array of byte);
var
  Con     : ^TVTTAPI_INT_CON;
begin
  Con:=hCon;

  if (@Pass = nil) or (hCon = nil) or (@PassPC = nil) or (@PassT = nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  TVTT_CalcSessionKey(Con.Key,PassPC,Con.RandPC,PassT,Con.RandT);

  ZeroMemory(Con.RandPC,16);
  ZeroMemory(Con.RandT,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja odczytuje sektor z pamici nieulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer sektora do odczytania
    Adres na bufor na sektor
Wy:
    wypeniony bufor na sektor
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemRead(Pass : array of byte; hCon : Pointer; SNr : byte; var Buff : array of byte);
var
  Con     : ^TVTTAPI_INT_CON;
  Res     : Cardinal;
  OutDataA: array [0..15] of byte;
  OutDataB: array [0..15] of byte;
  InDataA : array [0..15] of byte;
  InDataB : array [0..15] of byte;
  InKey   : Cardinal;
begin
  Con:=hCon;

  if (@Pass=nil) or (hCon=nil) or (@Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  ZeroMemory(Buff,16);

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  inc(Con.OutKey);
  OutDataB[TVTTD_CMD]:=TVTTC_NVMEM_READ;
  OutDataB[TVTTD_PARAM1]:=SNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  CopyMemory(@Buff,@InDataA,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
  exit;
end;

{*****************************************************************************
Funkcja odczytuje sektor z pamici ulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer sektora do odczytania
    Adres na bufor na sektor
Wy:
    wypeniony bufor na sektor
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_RamRead(Pass : array of byte; hCon : Pointer; SNr : byte; var Buff : array of byte);
var
  Con     : ^TVTTAPI_INT_CON;
  Res     : Cardinal;
  OutDataA: array [0..15] of byte;
  OutDataB: array [0..15] of byte;
  InDataA : array [0..15] of byte;
  InDataB : array [0..15] of byte;
  InKey   : Cardinal;
begin
  Con:=hCon;

  if (@Pass=nil) or (hCon=nil) or (@Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  ZeroMemory(Buff,16);

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_VMEM_READ;
  OutDataB[TVTTD_PARAM1]:=SNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  CopyMemory(@Buff,@InDataA,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje sektor do pamici ulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer sektora do zapisania
    Adres na bufor z danymi do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_RamWrite(Pass : array of byte; hCon : Pointer; SNr : byte; Buff : array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) or (@Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  CopyMemory(@OutDataA,@Buff,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_VMEM_WRITE;
  OutDataB[TVTTD_PARAM1]:=SNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje sektor do pamici nieulotnej
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer sektora do zapisania
    Adres na bufor z danymi do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemWrite(Pass : array of byte; hCon : Pointer; SNr : byte; Buff : array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) or (@Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  CopyMemory(@OutDataA,@Buff,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_NVMEM_WRITE;
  OutDataB[TVTTD_PARAM1]:=SNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje sektor do pamici nieulotnej i zabezpiecza sektor przed zapisem
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer sektora do zapisania
    Adres na bufor z danymi do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemWriteProtect(Pass : array of byte; hCon : Pointer; SNr : byte; Buff : array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) or (@Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  CopyMemory(@OutDataA,@Buff,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_NVMEM_WRITE_PROTECT;
  OutDataB[TVTTD_PARAM1]:=SNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zabezpiecza sektor w pamieci nieulotnej przed zapisem
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer sektora do zabezpieczenia
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_MemProtect(Pass : array of byte; hCon : Pointer; SNr : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_NVMEM_PROTECT;
  OutDataB[TVTTD_PARAM1]:=SNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja szyfruje blok danych o rozmiarze szesnastu bajtw przy uyciu Tokenu
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do zaszyfrowania
    Adres na bufor z danymi do zaszyfrowania
Wy:
    zaszyfrowane dane w buforze
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Encrypt(Pass : array of byte; hCon : Pointer; PNr : byte; var Buff : array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) or (@Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  CopyMemory(@OutDataA,@Buff,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_ENCRYPT;
  OutDataB[TVTTD_PARAM1]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      TVTTAPI_Int_SetLastError(InDataB[TVTTD_STATUS]);
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  CopyMemory(@Buff,@InDataA,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja deszyfruje blok szesnastu bajtw danych przy uyciu Tokenu
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do zaszyfrowania
    Adres na bufor z danymi do zaszyfrowania
Wy:
    zaszyfrowane dane w buforze
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_Decrypt(Pass : array of byte; hCon : Pointer; PNr : byte; var Buff: array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  CopyMemory(@OutDataA,@Buff,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_Decrypt;
  OutDataB[TVTTD_PARAM1]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  CopyMemory(@Buff,@InDataA,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja odczytuje wartoc z licznika uniwersalnego
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do odczytania
Wy:
    Warto odczytana z licznika
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_CounterRead(Pass : array of byte; hCon : Pointer; CNr : byte) : Cardinal;
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=0;
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_READ;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      result:=0;
      exit;
    end;

  Res:=InDataA[1];
  Res:=Res shl 8;
  Res:=Res or InDataA[0];

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
  result:=Res;
end;

{*****************************************************************************
Funkcja zwiksza licznik i odczytuje jego warto
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do odczytania
Wy:
    Warto odczytana z licznika
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_CounterIncRead(Pass : array of byte; hCon : Pointer; CNr : byte) : Cardinal;
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=0;
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_INC_READ;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      result:=0;
      exit;
    end;

  Res:=InDataA[1];
  Res:=Res shl 8;
  Res:=Res or InDataA[0];

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
  result:=Res;
end;

{*****************************************************************************
Funkcja zmniejsza licznik i odczytuje jego warto
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do odczytania
Wy:
    Warto odczytana z licznika
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_CounterDecRead(Pass : array of byte; hCon : Pointer; CNr : byte) : Cardinal;
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=0;
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_DEC_READ;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      result:=0;
      exit;
    end;

  Res:=InDataA[1];
  Res:=Res shl 8;
  Res:=Res or InDataA[0];

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
  result:=Res;
end;

{*****************************************************************************
Funkcja zwiksza licznik,odczytuje jego zawartoc i odsa ja zaszyfrowan podanym hasem
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do odczytania
    Numer hasa ktrym ma nastpi szyfrowanie
    Adres na bufor na dane
Wy:
    wypeniony bufor na dane
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterIncEncryptRead(Pass : array of byte; hCon : Pointer; CNr, PNr : byte; var Buff : array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  ZeroMemory(Buff,16);

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_INC_ENCRYPT_READ;
  OutDataB[TVTTD_PARAM1]:=CNr;
  OutDataB[TVTTD_PARAM2]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  CopyMemory(@Buff,@InDataA,16);

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje warto do licznika uniwersalnego
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do zapisania
    Wartoc do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterWrite(Pass : array of byte; hCon : Pointer; CNr : byte; Value : Cardinal);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=byte(Value);
  Value:=Value shr 8;
  OutDataA[1]:=byte(Value);
  Value:=0;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_WRITE;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje dwa bajty konfiguracji licznika
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do zapisania konfiguracji
    Wartoc pierwszego bajtu konfiguracji
    Wartoc drugiego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterConfigWrite(Pass : array of byte; hCon : Pointer; CNr : byte; Config0,Config1 : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=Config0;
  OutDataA[1]:=Config1;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_WRITE_CONFIG;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja ustawia bity w dwuch bajtach konfiguracji licznika
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterConfigSet(Pass : array of byte; hCon : Pointer; CNr : byte; Config0,Config1 : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=Config0;
  OutDataA[1]:=Config1;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_SET_CONFIG;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja kasuje bity w dwuch bajtach konfiguracji licznika
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer licznika do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_CounterConfigClear(Pass : array of byte; hCon : Pointer; CNr : byte; Config0,Config1 : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=Config0;
  OutDataA[1]:=Config1;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_CNT_CLR_CONFIG;
  OutDataB[TVTTD_PARAM1]:=CNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje haso do tablicy hase
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer pozycji w tablicy hase
    Adres na bufor z hasem do zapisania
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassWrite(Pass : array of byte; hCon : Pointer; PNr : byte; var Buff : array of byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  CopyMemory(@OutDataA,@Buff,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_PASS_WRITE;
  OutDataB[TVTTD_PARAM1]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zapisuje dwa bajty konfiguracji hasa
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do zapisania konfiguracji
    Wartoc pierwszego bajtu konfiguracji
    Wartoc drugiego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassConfigWrite(Pass : array of byte; hCon : Pointer; PNr : byte; Config0, Config1 : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=Config0;
  OutDataA[1]:=Config1;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_PASS_WRITE_CONFIG;
  OutDataB[TVTTD_PARAM1]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja ustawia bity w dwuch bajtach konfiguracji hasa
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassConfigSet(Pass : array of byte; hCon : Pointer; PNr : byte; Config0, Config1 : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=Config0;
  OutDataA[1]:=Config1;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_PASS_SET_CONFIG;
  OutDataB[TVTTD_PARAM1]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja kasuje bity w dwuch bajtach konfiguracji hasa
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do zapisania konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
    Maska bitowa pierwszego bajtu konfiguracji
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_PassConfigClear(Pass : array of byte; hCon : Pointer; PNr : byte; Config0, Config1 : byte);
var
  Con       : ^TVTTAPI_INT_CON;
  Res       : Cardinal;
  OutDataA  : array [0..15] of byte;
  OutDataB  : array [0..15] of byte;
  InDataA   : array [0..15] of byte;
  InDataB   : array [0..15] of byte;
  InKey     : Cardinal;
begin
  Con:=hCon;
  if (@Pass=nil) or (hCon=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;

  TVTTAPI_Int_DecodeCon(Pass,Con);

  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  OutDataA[0]:=Config0;
  OutDataA[1]:=Config1;

  Con.OutKey:=Con.OutKey + 1;
  OutDataB[TVTTD_CMD]:=TVTTC_PASS_CLR_CONFIG;
  OutDataB[TVTTD_PARAM1]:=PNr;

  TVTT_CalcAuth(OutDataA,OutDataB);
  TVTT_CryptBlock(OutDataA,Con.OutKey,Con.Key,TVTT_BLOCK_A,TVTT_PC2T);
  TVTT_CryptBlock(OutDataB,Con.OutKey,Con.Key,TVTT_BLOCK_B,TVTT_PC2T);
  Res:=TVTT_Talk(Con.hToken,5,Con.OutKey,@OutDataA,@OutDataB,@InKey,@InDataA,@InDataB);
  TVTT_CryptBlock(InDataA,InKey,Con.Key,TVTT_BLOCK_A,TVTT_T2PC);
  TVTT_CryptBlock(InDataB,InKey,Con.Key,TVTT_BLOCK_B,TVTT_T2PC);

  if (TVTTAPI_Int_CheckError(Con,Res,InKey,InDataA,InDataB)>0) then
    begin
      InKey:=0;
      ZeroMemory(OutDataA,16);
      ZeroMemory(OutDataB,16);
      ZeroMemory(InDataA,16);
      ZeroMemory(InDataB,16);
      TVTTAPI_Int_CodeCon(Pass,Con);
      exit;
    end;

  TVTTAPI_Int_CodeCon(Pass,Con);

  InKey:=0;
  ZeroMemory(OutDataA,16);
  ZeroMemory(OutDataB,16);
  ZeroMemory(InDataA,16);
  ZeroMemory(InDataB,16);

  SetLastError(ERROR_SUCCESS);
end;

{*****************************************************************************
Funkcja zwalnia pami zajt przez funkcje szyfrujc dane
    (funkcje szyfrujce bloki o dowolnym rozmiarze)
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do szyfrowania
    Adres na blok zwrcony przez funkcj szyfrujc
    Rozmiar bloku zwrcony przez funkcj szyfrujc
Wy:
    ustawiony "LastError"
*****************************************************************************}
procedure TVTTAPI_FreeBlock(Buff : Pointer; Size : Cardinal);
begin
  if (@Buff=nil) or (Size<(3*16)) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      exit;
    end;
  ZeroMemory(PByte(Buff)^,Size);
  GlobalFree(Cardinal(Buff));
end;

{*****************************************************************************
Funkcja szyfruje blok danych o dowolnym rozmiarze
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do szyfrowania
    Adres na blok do zaszyfrowania
    Rozmiar bloku do zaszyfrowania
    Adres na zmienn do ktrej zostanie wpisany rozmiar bloku po zaszyfrowaniu
Wy:
    Adres na zaszyfrowany blok danych
    Rozmiar bloku zaszyfrowanego
    ustawiony "LastError"
******************************************************************************}
function TVTTAPI_EncryptBlock(Pass : array of byte; hCon : Pointer; PNr : byte; Buff : Pointer; Size : Cardinal; var OutSize : Cardinal) : Pointer;
var
  n,i : Cardinal;
  OSize : Cardinal;
  OBuff : Pointer;
  Key   : array [0..15] of byte;

begin
  if((@Pass=nil)or(hCon=nil)or(@Buff=nil)or(Size=0)) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=nil;
      exit;
    end;

  OSize:=Size div 16;
  if (Size mod 16)>0 then OSize:=OSize + 1;
  OSize:=OSize + 4;
  OSize:=OSize * 16;

  OBuff:=Pointer(GlobalAlloc(GPTR,OSize));
  if (OBuff=nil) then
    begin
      result:=nil;
      exit;
    end;

  TVTT_Rand(PByte(Cardinal(OBuff) + (0*16))^);
  TVTT_Rand(PByte(Cardinal(OBuff) + (2*16))^);
  TVTT_Rand(PByte(Cardinal(OBuff) + (3*16))^);

  (PByte(Cardinal(OBuff)+(1*16)+0))^:=(Size shr 0) and $FF;
  (PByte(Cardinal(OBuff)+(1*16)+1))^:=(Size shr 8) and $FF;
  (PByte(Cardinal(OBuff)+(1*16)+2))^:=(Size shr 16)and $FF;
  (PByte(Cardinal(OBuff)+(1*16)+3))^:=(Size shr 24)and $FF;

  (PByte(Cardinal(OBuff)+(1*16)+4))^:=(OSize shr 0)and $FF;
  (PByte(Cardinal(OBuff)+(1*16)+5))^:=(OSize shr 8)and $FF;
  (PByte(Cardinal(OBuff)+(1*16)+6))^:=(OSize shr 16)and $FF;
  (PByte(Cardinal(OBuff)+(1*16)+7))^:=(OSize shr 24)and $FF;

  CopyMemory(PByte(Cardinal(OBuff)+(4*16)),Buff,Size);

  CopyMemory(@Key,PByte(Cardinal(OBuff)+(0*16)),16);

  TVTTAPI_Encrypt(Pass,hCon,PNr,Key);
  if (GetLastError()<>ERROR_SUCCESS) then
    begin
      ZeroMemory(PByte(Cardinal(OBuff))^,OSize);
      GlobalFree(Cardinal(OBuff));
      OBuff:=nil;
      OSize:=0;
      result:=nil;
      exit;
    end;

  rijndaelEncrypt(Key,PByte(Cardinal(OBuff) + (1*16))^);

  CopyMemory(@Key,PByte(Cardinal(OBuff)+(2*16)),16);
  TVTTAPI_Encrypt(Pass,hCon,PNr,Key);
  if (GetLastError()<>ERROR_SUCCESS) then
    begin
      ZeroMemory(PByte(Cardinal(OBuff))^,OSize);
      GlobalFree(PByte(Cardinal(OBuff))^);
      OBuff:=nil;
      OSize:=0;
      result:=nil;
      exit;
    end;

  n:=4*16;
  while (n<OSize) do
    begin
      i:=0;
      while (i<16) do
          begin
            PByte(Cardinal(OBuff)+(n+i))^:=PByte(Cardinal(OBuff)+(n+i))^ xor PByte(Cardinal(OBuff)+(n+i-16))^;
            inc(i);
          end;
      rijndaelEncrypt(Key,PByte(Cardinal(OBuff)+n)^);
      n:=n+16;
    end;

  SetLastError(ERROR_SUCCESS);
  OutSize:=OSize;
  result:=OBuff;
end;

{*****************************************************************************
Funkcja rozszyfruje wczeniej zasszyfrowany blok danych o dowolnym rozmiarze
We:
    Adres na 16-bajtowy klucz do poczenia
    Uchwyt na poczenie
    Numer hasa do szyfrowania
    Adres na blok do rozszyfrowania
    Adres na zmienn do ktrej zostanie wpisany rozmiar bloku po rozszyfrowaniu
Wy:
    Adres na rozszyfrowany blok danych
    Rozmiar bloku rozszyfrowanego
    ustawiony "LastError"
*****************************************************************************}
function TVTTAPI_DecryptBlock(Pass : array of byte; hCon : Pointer; PNr : byte; Buff : Pointer; var OutSize : Cardinal) : Pointer;
var
  n,i,t : Cardinal;
  Size  : Cardinal;
  TSize : Cardinal;
  OBuff : Pointer;
  TBuff : Pointer;
  Key   : array [0..15] of byte;
  Tmp   : array [0..15] of byte;
begin
  if (@Pass=nil) or (hCon=nil) or (Buff=nil) then
    begin
      SetLastError(ERROR_INVALID_DATA);
      result:=nil;
      exit;
    end;

  CopyMemory(@Key,PByte(Cardinal(Buff)+(0*16)),16);
  TVTTAPI_Encrypt(Pass,hCon,PNr,Key);
  if (GetLastError()<>ERROR_SUCCESS) then
    begin
      ZeroMemory(Key,16);
      ZeroMemory(Tmp,16);
      OBuff:=nil;
      TSize:=0;
      result:=nil;
      exit;
    end;

  CopyMemory(@Tmp,PByte(Cardinal(Buff)+(1*16)),16);
  rijndaelDecrypt(Key,Tmp);

  TSize:=Tmp[7];
  TSize:=TSize shl 8;
  TSize:=TSize or Tmp[6];
  TSize:=TSize shl 8;
  TSize:=TSize or Tmp[5];
  TSize:=TSize shl 8;
  TSize:=TSize or Tmp[4];

  Size:=Tmp[3];
  Size:=Size shl 8;
  Size:=Size or Tmp[2];
  Size:=Size shl 8;
  Size:=Size or Tmp[1];
  Size:=Size shl 8;
  Size:=Size or Tmp[0];

  TBuff:=Pointer(GlobalAlloc(GPTR,(TSize-(3*16))));
  if (TBuff=nil) then
    begin
      ZeroMemory(Key,16);
      ZeroMemory(Tmp,16);
      result:=nil;
      exit;
    end;

  OBuff:=Pointer(GlobalAlloc(GPTR,Size));
  if (OBuff=nil) then
    begin
      t:=GetLastError();
      ZeroMemory(Key,16);
      ZeroMemory(Tmp,16);
      GlobalFree(Cardinal(TBuff));
      SetLastError(t);
      result:=nil;
      exit;
    end;

  CopyMemory(TBuff,PByte(Cardinal(Buff)+(3*16)),TSize-(3*16));

  CopyMemory(@Key,PByte(Cardinal(Buff)+(2*16)),16);
  TVTTAPI_Encrypt(Pass,hCon,PNr,Key);
  if (GetLastError()<>ERROR_SUCCESS) then
    begin
      ZeroMemory(PByte(OBuff)^,Size);
      GlobalFree(Cardinal(OBuff));
      ZeroMemory(PByte(TBuff)^,TSize-(3*16));
      GlobalFree(Cardinal(TBuff));
      ZeroMemory(Key,16);
      ZeroMemory(Tmp,16);
      OBuff:=nil;
      TSize:=0;
      result:=nil;
      exit;
    end;

  n:=TSize-(3*16)-16;
  while (n>15) do
    begin
      rijndaelDecrypt(Key,PByte(Cardinal(TBuff)+n)^);

      i:=0;
      while (i<16) do
        begin
          PByte(Cardinal(TBuff)+(n+i))^:=PByte(Cardinal(TBuff)+(n+i))^ xor PByte(Cardinal(TBuff)+(n+i-16))^;
          inc(i);
        end;

      n:=n-16;
    end;

  CopyMemory(OBuff,PByte(Cardinal(TBuff)+16),Size);
  ZeroMemory(PByte(TBuff)^,TSize-(3*16));
  TBuff:=nil;
  GlobalFree(Cardinal(TBuff));

  SetLastError(ERROR_SUCCESS);
  OutSize:=Size;
  result:=OBuff;
end;
end.
